searchData={"items":[{"type":"module","title":"Sinter.DSPEx","doc":"Integration helpers specifically designed for DSPEx framework usage.\n\nThis module provides utilities that bridge Sinter's validation capabilities\nwith DSPEx's dynamic programming and teleprompter optimization needs.","ref":"Sinter.DSPEx.html"},{"type":"function","title":"Sinter.DSPEx.create_signature/3","doc":"Creates a signature schema for DSPEx programs.\n\nA signature defines the input and output structure for a DSPEx program,\ncombining both into a single schema suitable for validation and optimization.","ref":"Sinter.DSPEx.html#create_signature/3"},{"type":"function","title":"Parameters - Sinter.DSPEx.create_signature/3","doc":"* `input_fields` - List of input field specifications\n  * `output_fields` - List of output field specifications\n  * `opts` - Schema options","ref":"Sinter.DSPEx.html#create_signature/3-parameters"},{"type":"function","title":"Returns - Sinter.DSPEx.create_signature/3","doc":"* A schema representing the complete program signature","ref":"Sinter.DSPEx.html#create_signature/3-returns"},{"type":"function","title":"Examples - Sinter.DSPEx.create_signature/3","doc":"iex> signature = Sinter.DSPEx.create_signature(\n    ...>   [\n    ...>     {:query, :string, [required: true]},\n    ...>     {:context, {:array, :string}, [optional: true]}\n    ...>   ],\n    ...>   [\n    ...>     {:answer, :string, [required: true]},\n    ...>     {:confidence, :float, [required: true, gteq: 0.0, lteq: 1.0]}\n    ...>   ],\n    ...>   title: \"QA Program Signature\"\n    ...> )\n    iex> fields = Sinter.Schema.fields(signature)\n    iex> Map.keys(fields)\n    [:query, :context, :answer, :confidence]","ref":"Sinter.DSPEx.html#create_signature/3-examples"},{"type":"function","title":"Sinter.DSPEx.optimize_schema_from_failures/3","doc":"Optimizes a schema based on validation failure patterns.\n\nAnalyzes common validation failures to suggest schema improvements,\nuseful for teleprompter optimization loops.","ref":"Sinter.DSPEx.html#optimize_schema_from_failures/3"},{"type":"function","title":"Parameters - Sinter.DSPEx.optimize_schema_from_failures/3","doc":"* `original_schema` - The current schema\n  * `failure_examples` - List of data that failed validation\n  * `opts` - Optimization options","ref":"Sinter.DSPEx.html#optimize_schema_from_failures/3-parameters"},{"type":"function","title":"Options - Sinter.DSPEx.optimize_schema_from_failures/3","doc":"* `:relaxation_strategy` - How to relax constraints (`:conservative`, `:moderate`, `:aggressive`)\n  * `:add_missing_fields` - Whether to add commonly missing fields as optional","ref":"Sinter.DSPEx.html#optimize_schema_from_failures/3-options"},{"type":"function","title":"Returns - Sinter.DSPEx.optimize_schema_from_failures/3","doc":"* `{:ok, optimized_schema, suggestions}` or `{:error, reason}`","ref":"Sinter.DSPEx.html#optimize_schema_from_failures/3-returns"},{"type":"function","title":"Sinter.DSPEx.prepare_for_llm/3","doc":"Prepares a schema for specific LLM provider optimization.\n\nGenerates both the JSON Schema and metadata needed for optimal\nstructured output with different LLM providers.","ref":"Sinter.DSPEx.html#prepare_for_llm/3"},{"type":"function","title":"Parameters - Sinter.DSPEx.prepare_for_llm/3","doc":"* `schema` - The Sinter schema\n  * `provider` - The target LLM provider\n  * `opts` - Provider-specific options","ref":"Sinter.DSPEx.html#prepare_for_llm/3-parameters"},{"type":"function","title":"Returns - Sinter.DSPEx.prepare_for_llm/3","doc":"* Map with JSON Schema and provider-specific metadata","ref":"Sinter.DSPEx.html#prepare_for_llm/3-returns"},{"type":"function","title":"Examples - Sinter.DSPEx.prepare_for_llm/3","doc":"iex> schema = Sinter.Schema.define([{:name, :string, [required: true]}])\n    iex> result = Sinter.DSPEx.prepare_for_llm(schema, :openai)\n    iex> result.json_schema[\"additionalProperties\"]\n    false\n    iex> result.provider\n    :openai","ref":"Sinter.DSPEx.html#prepare_for_llm/3-examples"},{"type":"function","title":"Sinter.DSPEx.validate_llm_output/4","doc":"Validates LLM output and enhances errors with debugging context.\n\nThis is the primary validation function for DSPEx programs, combining\nstandard validation with LLM-specific error enhancement.","ref":"Sinter.DSPEx.html#validate_llm_output/4"},{"type":"function","title":"Parameters - Sinter.DSPEx.validate_llm_output/4","doc":"* `schema` - The validation schema\n  * `llm_output` - Raw output from LLM\n  * `original_prompt` - The prompt sent to the LLM\n  * `opts` - Validation options","ref":"Sinter.DSPEx.html#validate_llm_output/4-parameters"},{"type":"function","title":"Returns - Sinter.DSPEx.validate_llm_output/4","doc":"* `{:ok, validated_data}` or `{:error, enhanced_errors}`","ref":"Sinter.DSPEx.html#validate_llm_output/4-returns"},{"type":"function","title":"Examples - Sinter.DSPEx.validate_llm_output/4","doc":"iex> schema = Sinter.Schema.define([{:name, :string, [required: true]}])\n    iex> llm_output = %{\"age\" => 30}  # missing required name\n    iex> prompt = \"Generate a user profile\"\n    iex> {:error, errors} = Sinter.DSPEx.validate_llm_output(schema, llm_output, prompt)\n    iex> List.first(errors).context.prompt\n    \"Generate a user profile\"","ref":"Sinter.DSPEx.html#validate_llm_output/4-examples"},{"type":"module","title":"Sinter.Performance","doc":"Performance monitoring and optimization utilities for Sinter.\n\nThis module provides tools for monitoring validation performance and\noptimizing schemas for high-throughput scenarios common in DSPEx.","ref":"Sinter.Performance.html"},{"type":"function","title":"Sinter.Performance.analyze_memory_usage/2","doc":"Analyzes memory usage during validation.\n\nUseful for optimizing memory consumption in long-running DSPEx programs.","ref":"Sinter.Performance.html#analyze_memory_usage/2"},{"type":"function","title":"Parameters - Sinter.Performance.analyze_memory_usage/2","doc":"* `schema` - The schema to analyze\n  * `dataset` - Sample data for analysis","ref":"Sinter.Performance.html#analyze_memory_usage/2-parameters"},{"type":"function","title":"Returns - Sinter.Performance.analyze_memory_usage/2","doc":"* Map with memory usage statistics","ref":"Sinter.Performance.html#analyze_memory_usage/2-returns"},{"type":"function","title":"Sinter.Performance.benchmark_validation/3","doc":"Benchmarks validation performance for a schema and dataset.\n\nReturns timing information useful for optimizing DSPEx programs.","ref":"Sinter.Performance.html#benchmark_validation/3"},{"type":"function","title":"Parameters - Sinter.Performance.benchmark_validation/3","doc":"* `schema` - The schema to benchmark\n  * `dataset` - Sample data for benchmarking\n  * `opts` - Benchmark options","ref":"Sinter.Performance.html#benchmark_validation/3-parameters"},{"type":"function","title":"Options - Sinter.Performance.benchmark_validation/3","doc":"* `:iterations` - Number of iterations to run (default: 1000)\n  * `:warmup` - Number of warmup iterations (default: 100)","ref":"Sinter.Performance.html#benchmark_validation/3-options"},{"type":"function","title":"Returns - Sinter.Performance.benchmark_validation/3","doc":"* Map with timing statistics","ref":"Sinter.Performance.html#benchmark_validation/3-returns"},{"type":"function","title":"Examples - Sinter.Performance.benchmark_validation/3","doc":"iex> schema = Sinter.Schema.define([{:id, :integer, [required: true]}])\n    iex> dataset = [%{\"id\" => 1}, %{\"id\" => 2}, %{\"id\" => 3}]\n    iex> stats = Sinter.Performance.benchmark_validation(schema, dataset)\n    iex> is_number(stats.avg_time_microseconds)\n    true","ref":"Sinter.Performance.html#benchmark_validation/3-examples"},{"type":"function","title":"Sinter.Performance.profile_schema_complexity/1","doc":"Profiles schema complexity for optimization recommendations.\n\nAnalyzes a schema to identify potential performance bottlenecks and\nsuggests optimizations for DSPEx usage.","ref":"Sinter.Performance.html#profile_schema_complexity/1"},{"type":"function","title":"Parameters - Sinter.Performance.profile_schema_complexity/1","doc":"* `schema` - The schema to profile","ref":"Sinter.Performance.html#profile_schema_complexity/1-parameters"},{"type":"function","title":"Returns - Sinter.Performance.profile_schema_complexity/1","doc":"* Map with complexity analysis and optimization suggestions","ref":"Sinter.Performance.html#profile_schema_complexity/1-returns"},{"type":"module","title":"Sinter","doc":"Unified schema definition, validation, and JSON generation for Elixir.\n\nSinter provides a focused, high-performance schema validation library designed\nspecifically for dynamic frameworks. It follows the \"One True Way\" principle:\n\n- **One way** to define schemas (unified core engine)\n- **One way** to validate data (single validation pipeline)\n- **One way** to generate JSON Schema (unified generator)","ref":"Sinter.html"},{"type":"module","title":"Quick Start - Sinter","doc":"# Define a schema\n    fields = [\n      {:name, :string, [required: true]},\n      {:age, :integer, [optional: true, gt: 0]}\n    ]\n    schema = Sinter.Schema.define(fields)\n\n    # Validate data\n    {:ok, validated} = Sinter.Validator.validate(schema, %{\n      name: \"Alice\",\n      age: 30\n    })\n\n    # Generate JSON Schema\n    json_schema = Sinter.JsonSchema.generate(schema)","ref":"Sinter.html#module-quick-start"},{"type":"module","title":"Dynamic Schema Creation - Sinter","doc":"Sinter supports dynamic schema creation for teleprompters and runtime optimization:\n\n    # Infer schema from examples (perfect for MIPRO teleprompter)\n    examples = [\n      %{\"name\" => \"Alice\", \"age\" => 30},\n      %{\"name\" => \"Bob\", \"age\" => 25}\n    ]\n    schema = Sinter.infer_schema(examples)\n\n    # Merge schemas for signature composition\n    input_schema = Sinter.Schema.define([{:query, :string, [required: true]}])\n    output_schema = Sinter.Schema.define([{:answer, :string, [required: true]}])\n    program_schema = Sinter.merge_schemas([input_schema, output_schema])","ref":"Sinter.html#module-dynamic-schema-creation"},{"type":"module","title":"Convenience Helpers - Sinter","doc":"This module provides convenient helper functions for common validation tasks\nthat internally use the core unified engine.","ref":"Sinter.html#module-convenience-helpers"},{"type":"module","title":"Design Philosophy - Sinter","doc":"Sinter distills data validation to its pure essence, extracting the essential\npower from complex systems while eliminating unnecessary abstraction. It follows\nthree key principles:\n\n1. **Validation, Not Transformation** - Sinter validates data structure and constraints\n   but does not perform business logic transformations\n2. **Runtime-First Design** - Schemas are data structures that can be created and\n   modified at runtime, perfect for dynamic frameworks\n3. **Unified Core Engine** - All validation flows through a single, well-tested\n   pipeline for consistency and reliability","ref":"Sinter.html#module-design-philosophy"},{"type":"function","title":"Sinter.batch_validator_for/2","doc":"Creates a batch validator function for multiple type specifications.","ref":"Sinter.html#batch_validator_for/2"},{"type":"function","title":"Parameters - Sinter.batch_validator_for/2","doc":"* `field_specs` - List of field specifications or `{name, type_spec}` tuples\n  * `base_opts` - Base validation options","ref":"Sinter.html#batch_validator_for/2-parameters"},{"type":"function","title":"Examples - Sinter.batch_validator_for/2","doc":"iex> batch_validator = Sinter.batch_validator_for([\n    ...>   {:name, :string},\n    ...>   {:age, :integer}\n    ...> ])\n    iex> batch_validator.(%{name: \"Alice\", age: 30})\n    {:ok, %{name: \"Alice\", age: 30}}","ref":"Sinter.html#batch_validator_for/2-examples"},{"type":"function","title":"Sinter.infer_schema/2","doc":"Creates a schema by analyzing examples to infer field types and requirements.\n\nThis function is essential for DSPEx teleprompters like MIPRO that need to\ndynamically optimize schemas based on program execution examples.","ref":"Sinter.html#infer_schema/2"},{"type":"function","title":"Parameters - Sinter.infer_schema/2","doc":"* `examples` - List of maps representing example data\n  * `opts` - Schema creation options","ref":"Sinter.html#infer_schema/2-parameters"},{"type":"function","title":"Options - Sinter.infer_schema/2","doc":"* `:title` - Schema title\n  * `:description` - Schema description\n  * `:strict` - Whether to reject unknown fields (default: false)\n  * `:min_occurrence_ratio` - Minimum ratio for field to be considered required (default: 0.8)","ref":"Sinter.html#infer_schema/2-options"},{"type":"function","title":"Returns - Sinter.infer_schema/2","doc":"* `Schema.t()` - A schema inferred from the examples","ref":"Sinter.html#infer_schema/2-returns"},{"type":"function","title":"Examples - Sinter.infer_schema/2","doc":"iex> examples = [\n    ...>   %{\"name\" => \"Alice\", \"age\" => 30},\n    ...>   %{\"name\" => \"Bob\", \"age\" => 25},\n    ...>   %{\"name\" => \"Charlie\", \"age\" => 35}\n    ...> ]\n    iex> schema = Sinter.infer_schema(examples)\n    iex> fields = Sinter.Schema.fields(schema)\n    iex> fields[:name].type\n    :string\n    iex> fields[:age].type\n    :integer","ref":"Sinter.html#infer_schema/2-examples"},{"type":"function","title":"Algorithm - Sinter.infer_schema/2","doc":"1. **Field Discovery**: Find all unique field names across examples\n2. **Type Inference**: Determine the most common type for each field\n3. **Requirement Analysis**: Fields present in >= min_occurrence_ratio are required\n4. **Constraint Inference**: Infer basic constraints from value patterns","ref":"Sinter.html#infer_schema/2-algorithm"},{"type":"function","title":"Sinter.merge_schemas/2","doc":"Merges multiple schemas into a single schema.\n\nThis is useful for DSPEx signature composition where you need to combine\ninput and output schemas or merge component signatures.","ref":"Sinter.html#merge_schemas/2"},{"type":"function","title":"Parameters - Sinter.merge_schemas/2","doc":"* `schemas` - List of Schema.t() to merge\n  * `opts` - Optional schema configuration overrides","ref":"Sinter.html#merge_schemas/2-parameters"},{"type":"function","title":"Merge Rules - Sinter.merge_schemas/2","doc":"1. **Fields**: All fields from all schemas are included\n2. **Conflicts**: Later schemas override earlier ones for conflicting field definitions\n3. **Configuration**: First non-nil configuration value wins, except for `:strict` where last wins","ref":"Sinter.html#merge_schemas/2-merge-rules"},{"type":"function","title":"Examples - Sinter.merge_schemas/2","doc":"iex> input_schema = Sinter.Schema.define([\n    ...>   {:query, :string, [required: true]}\n    ...> ])\n    iex> output_schema = Sinter.Schema.define([\n    ...>   {:answer, :string, [required: true]},\n    ...>   {:confidence, :float, [optional: true]}\n    ...> ])\n    iex> merged = Sinter.merge_schemas([input_schema, output_schema])\n    iex> fields = Sinter.Schema.fields(merged)\n    iex> Map.keys(fields)\n    [:query, :answer, :confidence]","ref":"Sinter.html#merge_schemas/2-examples"},{"type":"type","title":"Sinter.schema/0","doc":"","ref":"Sinter.html#t:schema/0"},{"type":"function","title":"Sinter.validate_many/2","doc":"Validates multiple values efficiently against their respective type specifications.","ref":"Sinter.html#validate_many/2"},{"type":"function","title":"Parameters - Sinter.validate_many/2","doc":"* `type_value_pairs` - List of `{type_spec, value}` or `{field_name, type_spec, value}` tuples\n  * `opts` - Validation options","ref":"Sinter.html#validate_many/2-parameters"},{"type":"function","title":"Examples - Sinter.validate_many/2","doc":"iex> Sinter.validate_many([\n    ...>   {:string, \"hello\"},\n    ...>   {:integer, 42},\n    ...>   {:email, :string, \"test@example.com\", [format: ~r/@/]}\n    ...> ])\n    {:ok, [\"hello\", 42, \"test@example.com\"]}","ref":"Sinter.html#validate_many/2-examples"},{"type":"function","title":"Sinter.validate_type/3","doc":"Validates a single value against a type specification.\n\nThis is a convenient helper for one-off type validation that creates a\ntemporary schema internally and uses the unified validation engine.","ref":"Sinter.html#validate_type/3"},{"type":"function","title":"Parameters - Sinter.validate_type/3","doc":"* `type_spec` - The type specification to validate against\n  * `value` - The value to validate\n  * `opts` - Validation options","ref":"Sinter.html#validate_type/3-parameters"},{"type":"function","title":"Options - Sinter.validate_type/3","doc":"* `:coerce` - Enable type coercion (default: false)\n  * `:constraints` - Additional constraints to apply","ref":"Sinter.html#validate_type/3-options"},{"type":"function","title":"Returns - Sinter.validate_type/3","doc":"* `{:ok, validated_value}` on success\n  * `{:error, errors}` on validation failure","ref":"Sinter.html#validate_type/3-returns"},{"type":"function","title":"Examples - Sinter.validate_type/3","doc":"iex> Sinter.validate_type(:integer, \"42\", coerce: true)\n    {:ok, 42}\n\n    iex> Sinter.validate_type({:array, :string}, [\"hello\", \"world\"])\n    {:ok, [\"hello\", \"world\"]}\n\n    iex> {:error, [error]} = Sinter.validate_type(:string, 123)\n    iex> error.code\n    :type","ref":"Sinter.html#validate_type/3-examples"},{"type":"function","title":"Sinter.validate_value/4","doc":"Validates a single named value against a type specification.\n\nThis is a convenient helper for single field validation that creates a\ntemporary schema internally.","ref":"Sinter.html#validate_value/4"},{"type":"function","title":"Parameters - Sinter.validate_value/4","doc":"* `field_name` - Name for the field (used in error messages)\n  * `type_spec` - The type specification to validate against\n  * `value` - The value to validate\n  * `opts` - Validation options and constraints","ref":"Sinter.html#validate_value/4-parameters"},{"type":"function","title":"Examples - Sinter.validate_value/4","doc":"iex> Sinter.validate_value(:email, :string, \"test@example.com\",\n    ...>   constraints: [format: ~r/@/])\n    {:ok, \"test@example.com\"}\n\n    iex> Sinter.validate_value(:score, :integer, \"95\",\n    ...>   coerce: true, constraints: [gteq: 0, lteq: 100])\n    {:ok, 95}","ref":"Sinter.html#validate_value/4-examples"},{"type":"type","title":"Sinter.validation_opts/0","doc":"","ref":"Sinter.html#t:validation_opts/0"},{"type":"type","title":"Sinter.validation_result/0","doc":"","ref":"Sinter.html#t:validation_result/0"},{"type":"function","title":"Sinter.validator_for/2","doc":"Creates a reusable validation function for a specific type and constraints.\n\nReturns a function that can be used to validate multiple values against\nthe same specification efficiently.","ref":"Sinter.html#validator_for/2"},{"type":"function","title":"Parameters - Sinter.validator_for/2","doc":"* `type_spec` - The type specification\n  * `base_opts` - Base validation options and constraints","ref":"Sinter.html#validator_for/2-parameters"},{"type":"function","title":"Examples - Sinter.validator_for/2","doc":"iex> email_validator = Sinter.validator_for(:string,\n    ...>   constraints: [format: ~r/@/])\n    iex> email_validator.(\"test@example.com\")\n    {:ok, \"test@example.com\"}\n    iex> {:error, [error]} = email_validator.(\"invalid\")\n    iex> error.code\n    :format","ref":"Sinter.html#validator_for/2-examples"},{"type":"module","title":"Sinter.JsonSchema","doc":"Unified JSON Schema generation for Sinter.\n\nThis module provides the single JSON Schema generation engine that handles\nall JSON Schema creation in Sinter. It converts Sinter schemas into standard\nJSON Schema format with optional provider-specific optimizations.","ref":"Sinter.JsonSchema.html"},{"type":"module","title":"Features - Sinter.JsonSchema","doc":"- Standard JSON Schema generation\n- Provider-specific optimizations (OpenAI, Anthropic, etc.)\n- Reference resolution and flattening\n- Constraint mapping\n- Metadata preservation","ref":"Sinter.JsonSchema.html#module-features"},{"type":"module","title":"Usage - Sinter.JsonSchema","doc":"schema = Sinter.Schema.define([\n      {:name, :string, [required: true, min_length: 2]},\n      {:age, :integer, [optional: true, gt: 0]}\n    ])\n\n    # Basic JSON Schema generation\n    json_schema = Sinter.JsonSchema.generate(schema)\n\n    # Provider-specific optimization\n    openai_schema = Sinter.JsonSchema.generate(schema, optimize_for_provider: :openai)\n\n    # For specific providers\n    anthropic_schema = Sinter.JsonSchema.for_provider(schema, :anthropic)","ref":"Sinter.JsonSchema.html#module-usage"},{"type":"function","title":"Sinter.JsonSchema.for_provider/3","doc":"Generates a JSON Schema optimized for a specific LLM provider.\n\nThis is a convenience function that applies provider-specific optimizations\nand returns a schema ready for use with that provider's API.","ref":"Sinter.JsonSchema.html#for_provider/3"},{"type":"function","title":"Examples - Sinter.JsonSchema.for_provider/3","doc":"iex> openai_schema = Sinter.JsonSchema.for_provider(schema, :openai)\n    iex> anthropic_schema = Sinter.JsonSchema.for_provider(schema, :anthropic)","ref":"Sinter.JsonSchema.html#for_provider/3-examples"},{"type":"function","title":"Sinter.JsonSchema.generate/2","doc":"Generates a JSON Schema from a Sinter schema.\n\nThis is the core JSON Schema generation function that converts Sinter schemas\ninto standard JSON Schema format.","ref":"Sinter.JsonSchema.html#generate/2"},{"type":"function","title":"Parameters - Sinter.JsonSchema.generate/2","doc":"* `schema` - A Sinter schema created by `Sinter.Schema.define/2`\n  * `opts` - Generation options","ref":"Sinter.JsonSchema.html#generate/2-parameters"},{"type":"function","title":"Options - Sinter.JsonSchema.generate/2","doc":"* `:optimize_for_provider` - Apply provider-specific optimizations\n    - `:openai` - Optimize for OpenAI function calling\n    - `:anthropic` - Optimize for Anthropic tool use\n    - `:generic` - Standard JSON Schema (default)\n  * `:flatten` - Resolve all references inline (default: false)\n  * `:include_descriptions` - Include field descriptions (default: true)\n  * `:strict` - Override schema's strict setting for additionalProperties","ref":"Sinter.JsonSchema.html#generate/2-options"},{"type":"function","title":"Returns - Sinter.JsonSchema.generate/2","doc":"* JSON Schema map","ref":"Sinter.JsonSchema.html#generate/2-returns"},{"type":"function","title":"Examples - Sinter.JsonSchema.generate/2","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true, min_length: 2]},\n    ...>   {:age, :integer, [optional: true, gt: 0]}\n    ...> ], title: \"User Schema\")\n    iex> Sinter.JsonSchema.generate(schema)\n    %{\n      \"type\" => \"object\",\n      \"title\" => \"User Schema\",\n      \"properties\" => %{\n        \"name\" => %{\"type\" => \"string\", \"minLength\" => 2},\n        \"age\" => %{\"type\" => \"integer\", \"exclusiveMinimum\" => 0}\n      },\n      \"required\" => [\"name\"],\n      \"additionalProperties\" => false\n    }\n\n    # Provider-specific optimization\n    iex> Sinter.JsonSchema.generate(schema, optimize_for_provider: :openai)\n    %{\n      \"type\" => \"object\",\n      \"properties\" => %{...},\n      \"additionalProperties\" => false,\n      \"required\" => [\"name\"]\n    }","ref":"Sinter.JsonSchema.html#generate/2-examples"},{"type":"type","title":"Sinter.JsonSchema.generation_opts/0","doc":"","ref":"Sinter.JsonSchema.html#t:generation_opts/0"},{"type":"function","title":"Sinter.JsonSchema.validate_schema/2","doc":"Validates a JSON Schema for correctness and compatibility.","ref":"Sinter.JsonSchema.html#validate_schema/2"},{"type":"function","title":"Parameters - Sinter.JsonSchema.validate_schema/2","doc":"* `json_schema` - The JSON Schema to validate\n  * `opts` - Validation options","ref":"Sinter.JsonSchema.html#validate_schema/2-parameters"},{"type":"function","title":"Returns - Sinter.JsonSchema.validate_schema/2","doc":"* `:ok` if schema is valid\n  * `{:error, issues}` if problems are found","ref":"Sinter.JsonSchema.html#validate_schema/2-returns"},{"type":"module","title":"Sinter.Schema","doc":"Unified schema definition for Sinter.\n\nThis module provides the single, canonical way to define data validation schemas\nin Sinter. It follows the \"One True Way\" principle - all schema creation flows\nthrough `define/2`, whether at runtime or compile-time.","ref":"Sinter.Schema.html"},{"type":"module","title":"Basic Usage - Sinter.Schema","doc":"# Runtime schema definition\n    schema = Sinter.Schema.define([\n      {:name, :string, [required: true, min_length: 2]},\n      {:age, :integer, [optional: true, gt: 0]}\n    ])\n\n    # Compile-time schema definition\n    defmodule UserSchema do\n      use Sinter.Schema\n\n      use_schema do\n        field :name, :string, required: true, min_length: 2\n        field :age, :integer, optional: true, gt: 0\n      end\n    end","ref":"Sinter.Schema.html#module-basic-usage"},{"type":"module","title":"Field Specifications - Sinter.Schema","doc":"Each field is specified as a tuple: `{name, type_spec, options}`","ref":"Sinter.Schema.html#module-field-specifications"},{"type":"module","title":"Supported Options - Sinter.Schema","doc":"* `:required` - Field must be present (default: true)\n* `:optional` - Field may be omitted (default: false)\n* `:default` - Default value if field is missing (implies optional: true)\n* `:description` - Human-readable description\n* `:example` - Example value for documentation","ref":"Sinter.Schema.html#module-supported-options"},{"type":"module","title":"Constraints - Sinter.Schema","doc":"* `:min_length`, `:max_length` - For strings and arrays\n* `:gt`, `:gteq`, `:lt`, `:lteq` - For numbers\n* `:format` - Regex pattern for strings\n* `:choices` - List of allowed values","ref":"Sinter.Schema.html#module-constraints"},{"type":"module","title":"Schema Configuration - Sinter.Schema","doc":"* `:title` - Schema title for documentation\n* `:description` - Schema description\n* `:strict` - Reject unknown fields (default: false)\n* `:post_validate` - Custom validation function","ref":"Sinter.Schema.html#module-schema-configuration"},{"type":"type","title":"Sinter.Schema.config/0","doc":"","ref":"Sinter.Schema.html#t:config/0"},{"type":"function","title":"Sinter.Schema.config/1","doc":"Returns the schema configuration.","ref":"Sinter.Schema.html#config/1"},{"type":"function","title":"Examples - Sinter.Schema.config/1","doc":"iex> schema = Sinter.Schema.define([], title: \"Test Schema\")\n    iex> config = Sinter.Schema.config(schema)\n    iex> config.title\n    \"Test Schema\"","ref":"Sinter.Schema.html#config/1-examples"},{"type":"function","title":"Sinter.Schema.constraints/1","doc":"Extracts constraint information from schema fields.\n\nReturns a map of field names to their constraint lists, useful for\nteleprompter analysis and optimization.","ref":"Sinter.Schema.html#constraints/1"},{"type":"function","title":"Parameters - Sinter.Schema.constraints/1","doc":"* `schema` - A Sinter schema","ref":"Sinter.Schema.html#constraints/1-parameters"},{"type":"function","title":"Returns - Sinter.Schema.constraints/1","doc":"* A map of field_name => constraints_list","ref":"Sinter.Schema.html#constraints/1-returns"},{"type":"function","title":"Examples - Sinter.Schema.constraints/1","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true, min_length: 2, max_length: 50]},\n    ...>   {:score, :integer, [required: true, gt: 0, lteq: 100]}\n    ...> ])\n    iex> Sinter.Schema.constraints(schema)\n    %{\n      name: [min_length: 2, max_length: 50],\n      score: [gt: 0, lteq: 100]\n    }","ref":"Sinter.Schema.html#constraints/1-examples"},{"type":"function","title":"Sinter.Schema.define/2","doc":"Defines a schema from field specifications.\n\nThis is the unified entry point for all schema creation in Sinter.\nBoth runtime and compile-time schema definition ultimately use this function.","ref":"Sinter.Schema.html#define/2"},{"type":"function","title":"Parameters - Sinter.Schema.define/2","doc":"* `field_specs` - List of field specifications\n  * `opts` - Schema configuration options","ref":"Sinter.Schema.html#define/2-parameters"},{"type":"function","title":"Options - Sinter.Schema.define/2","doc":"* `:title` - Schema title for documentation\n  * `:description` - Schema description\n  * `:strict` - Reject unknown fields (default: false)\n  * `:post_validate` - Custom validation function","ref":"Sinter.Schema.html#define/2-options"},{"type":"function","title":"Examples - Sinter.Schema.define/2","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true, min_length: 2]},\n    ...>   {:age, :integer, [optional: true, gt: 0]}\n    ...> ], title: \"User Schema\")\n    iex> schema.config.title\n    \"User Schema\"\n\n    iex> schema.fields[:name].required\n    true","ref":"Sinter.Schema.html#define/2-examples"},{"type":"macro","title":"Sinter.Schema.field/3","doc":"Defines a field in the schema.\n\nUsed within `use_schema` blocks.","ref":"Sinter.Schema.html#field/3"},{"type":"macro","title":"Examples - Sinter.Schema.field/3","doc":"field :name, :string, required: true, min_length: 2\n    field :age, :integer, optional: true, gt: 0\n    field :active, :boolean, optional: true, default: true","ref":"Sinter.Schema.html#field/3-examples"},{"type":"type","title":"Sinter.Schema.field_definition/0","doc":"","ref":"Sinter.Schema.html#t:field_definition/0"},{"type":"type","title":"Sinter.Schema.field_spec/0","doc":"","ref":"Sinter.Schema.html#t:field_spec/0"},{"type":"function","title":"Sinter.Schema.field_types/1","doc":"Extracts field types from a schema for analysis and introspection.\n\nThis is useful for DSPEx teleprompters that need to analyze the structure\nof schemas for optimization purposes.","ref":"Sinter.Schema.html#field_types/1"},{"type":"function","title":"Parameters - Sinter.Schema.field_types/1","doc":"* `schema` - A Sinter schema","ref":"Sinter.Schema.html#field_types/1-parameters"},{"type":"function","title":"Returns - Sinter.Schema.field_types/1","doc":"* A map of field_name => type_spec","ref":"Sinter.Schema.html#field_types/1-returns"},{"type":"function","title":"Examples - Sinter.Schema.field_types/1","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true]},\n    ...>   {:tags, {:array, :string}, [optional: true]}\n    ...> ])\n    iex> Sinter.Schema.field_types(schema)\n    %{\n      name: :string,\n      tags: {:array, :string}\n    }","ref":"Sinter.Schema.html#field_types/1-examples"},{"type":"function","title":"Sinter.Schema.fields/1","doc":"Returns the field definitions map.","ref":"Sinter.Schema.html#fields/1"},{"type":"function","title":"Examples - Sinter.Schema.fields/1","doc":"iex> schema = Sinter.Schema.define([{:name, :string, [required: true]}])\n    iex> fields = Sinter.Schema.fields(schema)\n    iex> fields[:name].required\n    true","ref":"Sinter.Schema.html#fields/1-examples"},{"type":"function","title":"Sinter.Schema.info/1","doc":"Returns summary information about the schema.","ref":"Sinter.Schema.html#info/1"},{"type":"function","title":"Examples - Sinter.Schema.info/1","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true]},\n    ...>   {:age, :integer, [optional: true]}\n    ...> ], title: \"User Schema\")\n    iex> info = Sinter.Schema.info(schema)\n    iex> info.field_count\n    2\n    iex> info.title\n    \"User Schema\"","ref":"Sinter.Schema.html#info/1-examples"},{"type":"type","title":"Sinter.Schema.metadata/0","doc":"","ref":"Sinter.Schema.html#t:metadata/0"},{"type":"macro","title":"Sinter.Schema.option/2","doc":"Adds an option to the schema being defined.\n\nUsed within `use_schema` blocks.","ref":"Sinter.Schema.html#option/2"},{"type":"macro","title":"Examples - Sinter.Schema.option/2","doc":"option :title, \"User Schema\"\n    option :strict, true","ref":"Sinter.Schema.html#option/2-examples"},{"type":"function","title":"Sinter.Schema.optional_fields/1","doc":"Returns a list of optional field names.","ref":"Sinter.Schema.html#optional_fields/1"},{"type":"function","title":"Examples - Sinter.Schema.optional_fields/1","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true]},\n    ...>   {:age, :integer, [optional: true]}\n    ...> ])\n    iex> Sinter.Schema.optional_fields(schema)\n    [:age]","ref":"Sinter.Schema.html#optional_fields/1-examples"},{"type":"function","title":"Sinter.Schema.post_validate_fn/1","doc":"Returns the post-validation function if defined.","ref":"Sinter.Schema.html#post_validate_fn/1"},{"type":"function","title":"Examples - Sinter.Schema.post_validate_fn/1","doc":"iex> post_fn = fn data -> {:ok, data} end\n    iex> schema = Sinter.Schema.define([], post_validate: post_fn)\n    iex> Sinter.Schema.post_validate_fn(schema)\n    #Function<...>","ref":"Sinter.Schema.html#post_validate_fn/1-examples"},{"type":"function","title":"Sinter.Schema.required_fields/1","doc":"Returns a list of required field names.","ref":"Sinter.Schema.html#required_fields/1"},{"type":"function","title":"Examples - Sinter.Schema.required_fields/1","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true]},\n    ...>   {:age, :integer, [optional: true]}\n    ...> ])\n    iex> Sinter.Schema.required_fields(schema)\n    [:name]","ref":"Sinter.Schema.html#required_fields/1-examples"},{"type":"function","title":"Sinter.Schema.strict?/1","doc":"Returns true if the schema is in strict mode.","ref":"Sinter.Schema.html#strict?/1"},{"type":"function","title":"Examples - Sinter.Schema.strict?/1","doc":"iex> schema = Sinter.Schema.define([], strict: true)\n    iex> Sinter.Schema.strict?(schema)\n    true","ref":"Sinter.Schema.html#strict?/1-examples"},{"type":"type","title":"Sinter.Schema.t/0","doc":"","ref":"Sinter.Schema.html#t:t/0"},{"type":"macro","title":"Sinter.Schema.use_schema/1","doc":"Compile-time schema definition macro.\n\nThis macro provides a DSL for defining schemas at compile time.\nIt accumulates field and option definitions and creates a schema\nusing `define/2`.","ref":"Sinter.Schema.html#use_schema/1"},{"type":"macro","title":"Example - Sinter.Schema.use_schema/1","doc":"defmodule UserSchema do\n      use Sinter.Schema\n\n      use_schema do\n        option :title, \"User Schema\"\n        option :strict, true\n\n        field :name, :string, required: true, min_length: 2\n        field :age, :integer, optional: true, gt: 0\n        field :active, :boolean, optional: true, default: true\n      end\n    end\n\n    # The module will have a schema/0 function\n    UserSchema.schema()","ref":"Sinter.Schema.html#use_schema/1-example"},{"type":"module","title":"Sinter.Validator","doc":"The unified validation engine for Sinter.\n\nThis module provides the single validation pipeline that handles all types\nof validation in Sinter. It processes schemas created by `Sinter.Schema.define/2`\nand runs a clean, predictable validation process.","ref":"Sinter.Validator.html"},{"type":"module","title":"Validation Pipeline - Sinter.Validator","doc":"1. **Input Validation** - Ensure input is valid format\n2. **Required Field Check** - Verify all required fields are present\n3. **Field Validation** - Validate each field against its type and constraints\n4. **Strict Mode Check** - Reject unknown fields if strict mode enabled\n5. **Post Validation** - Run custom cross-field validation if configured","ref":"Sinter.Validator.html#module-validation-pipeline"},{"type":"module","title":"Design Philosophy - Sinter.Validator","doc":"The validator focuses purely on validation - it does NOT perform data transformation.\nAny data transformation should be explicit in your application code, keeping\nyour validation logic pure and your transformations visible.","ref":"Sinter.Validator.html#module-design-philosophy"},{"type":"module","title":"Usage - Sinter.Validator","doc":"schema = Sinter.Schema.define([\n      {:name, :string, [required: true, min_length: 2]},\n      {:age, :integer, [optional: true, gt: 0]}\n    ])\n\n    # Basic validation\n    {:ok, validated} = Sinter.Validator.validate(schema, %{name: \"Alice\", age: 30})\n\n    # With coercion\n    {:ok, validated} = Sinter.Validator.validate(schema, %{name: \"Alice\", age: \"30\"}, coerce: true)\n\n    # Batch validation\n    data_list = [%{name: \"Alice\", age: 30}, %{name: \"Bob\", age: 25}]\n    {:ok, validated_list} = Sinter.Validator.validate_many(schema, data_list)","ref":"Sinter.Validator.html#module-usage"},{"type":"function","title":"Sinter.Validator.validate/3","doc":"Validates data against a Sinter schema.\n\nThis is the core validation function that all other validation in Sinter\nultimately uses. It implements a clean, predictable pipeline.","ref":"Sinter.Validator.html#validate/3"},{"type":"function","title":"Parameters - Sinter.Validator.validate/3","doc":"* `schema` - A schema created by `Sinter.Schema.define/2`\n  * `data` - The data to validate (must be a map)\n  * `opts` - Validation options","ref":"Sinter.Validator.html#validate/3-parameters"},{"type":"function","title":"Options - Sinter.Validator.validate/3","doc":"* `:coerce` - Enable type coercion (default: false)\n  * `:strict` - Override schema's strict setting\n  * `:path` - Base path for error reporting (default: [])","ref":"Sinter.Validator.html#validate/3-options"},{"type":"function","title":"Returns - Sinter.Validator.validate/3","doc":"* `{:ok, validated_data}` - Validation succeeded\n  * `{:error, errors}` - List of validation errors","ref":"Sinter.Validator.html#validate/3-returns"},{"type":"function","title":"Examples - Sinter.Validator.validate/3","doc":"iex> schema = Sinter.Schema.define([\n    ...>   {:name, :string, [required: true]},\n    ...>   {:age, :integer, [optional: true, gt: 0]}\n    ...> ])\n    iex> Sinter.Validator.validate(schema, %{name: \"Alice\", age: 30})\n    {:ok, %{name: \"Alice\", age: 30}}\n\n    iex> Sinter.Validator.validate(schema, %{age: 30})\n    {:error, [%Sinter.Error{path: [:name], code: :required, ...}]}","ref":"Sinter.Validator.html#validate/3-examples"},{"type":"function","title":"Sinter.Validator.validate!/3","doc":"Validates data against a schema, raising an exception on failure.","ref":"Sinter.Validator.html#validate!/3"},{"type":"function","title":"Examples - Sinter.Validator.validate!/3","doc":"iex> validated = Sinter.Validator.validate!(schema, data)\n    %{name: \"Alice\", age: 30}\n\n    # Raises Sinter.ValidationError on failure","ref":"Sinter.Validator.html#validate!/3-examples"},{"type":"function","title":"Sinter.Validator.validate_many/3","doc":"Validates multiple data items against the same schema efficiently.","ref":"Sinter.Validator.html#validate_many/3"},{"type":"function","title":"Parameters - Sinter.Validator.validate_many/3","doc":"* `schema` - Schema to validate against\n  * `data_list` - List of data maps to validate\n  * `opts` - Validation options","ref":"Sinter.Validator.html#validate_many/3-parameters"},{"type":"function","title":"Returns - Sinter.Validator.validate_many/3","doc":"* `{:ok, validated_list}` if all validations succeed\n  * `{:error, errors_by_index}` if any validation fails","ref":"Sinter.Validator.html#validate_many/3-returns"},{"type":"function","title":"Examples - Sinter.Validator.validate_many/3","doc":"iex> data_list = [\n    ...>   %{name: \"Alice\", age: 30},\n    ...>   %{name: \"Bob\", age: 25}\n    ...> ]\n    iex> Sinter.Validator.validate_many(schema, data_list)\n    {:ok, [%{name: \"Alice\", age: 30}, %{name: \"Bob\", age: 25}]}","ref":"Sinter.Validator.html#validate_many/3-examples"},{"type":"function","title":"Sinter.Validator.validate_stream/3","doc":"Validates a stream of data maps against a schema with memory efficiency.\n\nThis function is designed for processing large datasets without loading\neverything into memory at once. Perfect for DSPEx teleprompter optimization\non large training sets.","ref":"Sinter.Validator.html#validate_stream/3"},{"type":"function","title":"Parameters - Sinter.Validator.validate_stream/3","doc":"* `schema` - A schema created by `Sinter.Schema.define/2`\n  * `data_stream` - An Enumerable of data maps to validate\n  * `opts` - Validation options","ref":"Sinter.Validator.html#validate_stream/3-parameters"},{"type":"function","title":"Returns - Sinter.Validator.validate_stream/3","doc":"* A stream of `{:ok, validated_data}` or `{:error, [Error.t()]}` tuples","ref":"Sinter.Validator.html#validate_stream/3-returns"},{"type":"function","title":"Examples - Sinter.Validator.validate_stream/3","doc":"iex> schema = Sinter.Schema.define([{:id, :integer, [required: true]}])\n    iex> data_stream = Stream.map(1..1000, &%{\"id\" => &1})\n    iex> results = Sinter.Validator.validate_stream(schema, data_stream)\n    iex> Enum.take(results, 3)\n    [\n      {:ok, %{id: 1}},\n      {:ok, %{id: 2}},\n      {:ok, %{id: 3}}\n    ]","ref":"Sinter.Validator.html#validate_stream/3-examples"},{"type":"function","title":"Memory Efficiency - Sinter.Validator.validate_stream/3","doc":"This function processes items one at a time and does not accumulate results,\nmaking it suitable for very large datasets that would not fit in memory.","ref":"Sinter.Validator.html#validate_stream/3-memory-efficiency"},{"type":"type","title":"Sinter.Validator.validation_opts/0","doc":"","ref":"Sinter.Validator.html#t:validation_opts/0"},{"type":"type","title":"Sinter.Validator.validation_result/0","doc":"","ref":"Sinter.Validator.html#t:validation_result/0"},{"type":"module","title":"Sinter.Error","doc":"Structured error representation for Sinter validation errors.\n\nThis module provides a consistent way to represent validation errors\nthroughout Sinter, including path information for nested data structures\nand machine-readable error codes.","ref":"Sinter.Error.html"},{"type":"function","title":"Sinter.Error.filter_by_code/2","doc":"Filters errors by error code.","ref":"Sinter.Error.html#filter_by_code/2"},{"type":"function","title":"Examples - Sinter.Error.filter_by_code/2","doc":"iex> errors = [\n    ...>   Sinter.Error.new([:name], :required, \"field is required\"),\n    ...>   Sinter.Error.new([:age], :type, \"expected integer\")\n    ...> ]\n    iex> Sinter.Error.filter_by_code(errors, :required)\n    [%Sinter.Error{path: [:name], code: :required, ...}]","ref":"Sinter.Error.html#filter_by_code/2-examples"},{"type":"function","title":"Sinter.Error.format/2","doc":"Formats an error into a human-readable string.","ref":"Sinter.Error.html#format/2"},{"type":"function","title":"Parameters - Sinter.Error.format/2","doc":"* `error` - The error to format\n  * `opts` - Formatting options","ref":"Sinter.Error.html#format/2-parameters"},{"type":"function","title":"Options - Sinter.Error.format/2","doc":"* `:include_path` - Include the path in the formatted message (default: true)\n  * `:path_separator` - Separator for path elements (default: \".\")","ref":"Sinter.Error.html#format/2-options"},{"type":"function","title":"Examples - Sinter.Error.format/2","doc":"iex> error = %Sinter.Error{\n    ...>   path: [:user, :email],\n    ...>   code: :format,\n    ...>   message: \"invalid email format\"\n    ...> }\n    iex> Sinter.Error.format(error)\n    \"user.email: invalid email format\"\n\n    iex> Sinter.Error.format(error, include_path: false)\n    \"invalid email format\"","ref":"Sinter.Error.html#format/2-examples"},{"type":"function","title":"Sinter.Error.format_errors/2","doc":"Formats multiple errors into a readable string.","ref":"Sinter.Error.html#format_errors/2"},{"type":"function","title":"Examples - Sinter.Error.format_errors/2","doc":"iex> errors = [\n    ...>   Sinter.Error.new([:name], :required, \"field is required\"),\n    ...>   Sinter.Error.new([:age], :type, \"expected integer\")\n    ...> ]\n    iex> Sinter.Error.format_errors(errors)\n    \"name: field is required\\nage: expected integer\"","ref":"Sinter.Error.html#format_errors/2-examples"},{"type":"function","title":"Sinter.Error.group_by_code/1","doc":"Groups errors by their error code.","ref":"Sinter.Error.html#group_by_code/1"},{"type":"function","title":"Examples - Sinter.Error.group_by_code/1","doc":"iex> errors = [\n    ...>   Sinter.Error.new([:name], :required, \"field is required\"),\n    ...>   Sinter.Error.new([:email], :required, \"field is required\"),\n    ...>   Sinter.Error.new([:age], :type, \"expected integer\")\n    ...> ]\n    iex> Sinter.Error.group_by_code(errors)\n    %{\n      required: [\n        %Sinter.Error{path: [:name], ...},\n        %Sinter.Error{path: [:email], ...}\n      ],\n      type: [\n        %Sinter.Error{path: [:age], ...}\n      ]\n    }","ref":"Sinter.Error.html#group_by_code/1-examples"},{"type":"function","title":"Sinter.Error.group_by_path/1","doc":"Groups errors by their path for easier processing.","ref":"Sinter.Error.html#group_by_path/1"},{"type":"function","title":"Examples - Sinter.Error.group_by_path/1","doc":"iex> errors = [\n    ...>   Sinter.Error.new([:user, :name], :required, \"field is required\"),\n    ...>   Sinter.Error.new([:user, :name], :min_length, \"too short\"),\n    ...>   Sinter.Error.new([:user, :email], :format, \"invalid format\")\n    ...> ]\n    iex> Sinter.Error.group_by_path(errors)\n    %{\n      [:user, :name] => [\n        %Sinter.Error{code: :required, ...},\n        %Sinter.Error{code: :min_length, ...}\n      ],\n      [:user, :email] => [\n        %Sinter.Error{code: :format, ...}\n      ]\n    }","ref":"Sinter.Error.html#group_by_path/1-examples"},{"type":"function","title":"Sinter.Error.new/4","doc":"Creates a new validation error.","ref":"Sinter.Error.html#new/4"},{"type":"function","title":"Parameters - Sinter.Error.new/4","doc":"* `path` - Path to the field that caused the error\n  * `code` - Machine-readable error code\n  * `message` - Human-readable error message\n  * `context` - Optional additional context information","ref":"Sinter.Error.html#new/4-parameters"},{"type":"function","title":"Examples - Sinter.Error.new/4","doc":"iex> Sinter.Error.new([:user, :email], :format, \"invalid email format\")\n    %Sinter.Error{\n      path: [:user, :email],\n      code: :format,\n      message: \"invalid email format\",\n      context: nil\n    }\n\n    iex> Sinter.Error.new(:name, :required, \"field is required\")\n    %Sinter.Error{path: [:name], code: :required, message: \"field is required\"}","ref":"Sinter.Error.html#new/4-examples"},{"type":"function","title":"Sinter.Error.summarize/1","doc":"Summarizes validation errors into a report.","ref":"Sinter.Error.html#summarize/1"},{"type":"function","title":"Examples - Sinter.Error.summarize/1","doc":"iex> errors = [\n    ...>   Sinter.Error.new([:name], :required, \"field is required\"),\n    ...>   Sinter.Error.new([:age], :type, \"expected integer\"),\n    ...>   Sinter.Error.new([:email], :format, \"invalid email format\")\n    ...> ]\n    iex> Sinter.Error.summarize(errors)\n    %{\n      total_errors: 3,\n      error_codes: [:required, :type, :format],\n      affected_paths: [[:name], [:age], [:email]],\n      by_code: %{\n        required: 1,\n        type: 1,\n        format: 1\n      }\n    }","ref":"Sinter.Error.html#summarize/1-examples"},{"type":"type","title":"Sinter.Error.t/0","doc":"","ref":"Sinter.Error.html#t:t/0"},{"type":"function","title":"Sinter.Error.to_map/1","doc":"Converts an error to a map representation.\n\nUseful for JSON serialization or API responses.","ref":"Sinter.Error.html#to_map/1"},{"type":"function","title":"Examples - Sinter.Error.to_map/1","doc":"iex> error = Sinter.Error.new([:user, :email], :format, \"invalid email format\")\n    iex> Sinter.Error.to_map(error)\n    %{\n      \"path\" => [\"user\", \"email\"],\n      \"code\" => \"format\",\n      \"message\" => \"invalid email format\"\n    }","ref":"Sinter.Error.html#to_map/1-examples"},{"type":"function","title":"Sinter.Error.to_maps/1","doc":"Converts multiple errors to map representations.","ref":"Sinter.Error.html#to_maps/1"},{"type":"function","title":"Examples - Sinter.Error.to_maps/1","doc":"iex> errors = [\n    ...>   Sinter.Error.new([:name], :required, \"field is required\"),\n    ...>   Sinter.Error.new([:age], :type, \"expected integer\")\n    ...> ]\n    iex> Sinter.Error.to_maps(errors)\n    [\n      %{\"path\" => [\"name\"], \"code\" => \"required\", \"message\" => \"field is required\"},\n      %{\"path\" => [\"age\"], \"code\" => \"type\", \"message\" => \"expected integer\"}\n    ]","ref":"Sinter.Error.html#to_maps/1-examples"},{"type":"function","title":"Sinter.Error.with_context/4","doc":"Creates a new error with additional context information.","ref":"Sinter.Error.html#with_context/4"},{"type":"function","title":"Examples - Sinter.Error.with_context/4","doc":"iex> context = %{expected: \"string\", actual: \"integer\", value: 42}\n    iex> Sinter.Error.with_context([:age], :type, \"expected string\", context)\n    %Sinter.Error{\n      path: [:age],\n      code: :type,\n      message: \"expected string\",\n      context: %{expected: \"string\", actual: \"integer\", value: 42}\n    }","ref":"Sinter.Error.html#with_context/4-examples"},{"type":"function","title":"Sinter.Error.with_llm_context/3","doc":"Adds LLM debugging context to a validation error.\n\nThis function enhances errors with information about the LLM response and\noriginal prompt, making it easier to debug validation failures in DSPEx\nprograms.","ref":"Sinter.Error.html#with_llm_context/3"},{"type":"function","title":"Parameters - Sinter.Error.with_llm_context/3","doc":"* `error` - The original validation error\n  * `llm_response` - The raw response from the LLM\n  * `prompt` - The original prompt sent to the LLM","ref":"Sinter.Error.html#with_llm_context/3-parameters"},{"type":"function","title":"Returns - Sinter.Error.with_llm_context/3","doc":"* Enhanced error with LLM context information","ref":"Sinter.Error.html#with_llm_context/3-returns"},{"type":"function","title":"Examples - Sinter.Error.with_llm_context/3","doc":"iex> error = Sinter.Error.new([:name], :required, \"field is required\")\n    iex> llm_response = %{\"age\" => 30}  # missing name field\n    iex> prompt = \"Generate a user profile with name and age\"\n    iex> enhanced = Sinter.Error.with_llm_context(error, llm_response, prompt)\n    iex> enhanced.context.llm_response\n    %{\"age\" => 30}\n    iex> enhanced.context.prompt\n    \"Generate a user profile with name and age\"","ref":"Sinter.Error.html#with_llm_context/3-examples"},{"type":"module","title":"Sinter.Types","doc":"Core type system for Sinter validation.\n\nThis module defines the type specifications supported by Sinter and provides\nthe fundamental validation and coercion functions that power the unified\nvalidation engine.","ref":"Sinter.Types.html"},{"type":"module","title":"Supported Types - Sinter.Types","doc":"* **Primitive Types**: `:string`, `:integer`, `:float`, `:boolean`, `:atom`, `:any`, `:map`\n* **Array Types**: `{:array, inner_type}`\n* **Union Types**: `{:union, [type1, type2, ...]}`\n* **Tuple Types**: `{:tuple, [type1, type2, ...]}`\n* **Map Types**: `{:map, key_type, value_type}` or `:map`","ref":"Sinter.Types.html#module-supported-types"},{"type":"module","title":"Type Validation - Sinter.Types","doc":"The `validate/3` function is the core validation engine:\n\n    iex> Sinter.Types.validate(:string, \"hello\", [])\n    {:ok, \"hello\"}\n\n    iex> Sinter.Types.validate(:integer, \"not a number\", [])\n    {:error, [%Sinter.Error{code: :type, ...}]}","ref":"Sinter.Types.html#module-type-validation"},{"type":"module","title":"Type Coercion - Sinter.Types","doc":"The `coerce/2` function provides safe type conversion:\n\n    iex> Sinter.Types.coerce(:integer, \"42\")\n    {:ok, 42}\n\n    iex> Sinter.Types.coerce(:integer, \"not a number\")\n    {:error, [%Sinter.Error{code: :coercion, ...}]}","ref":"Sinter.Types.html#module-type-coercion"},{"type":"function","title":"Sinter.Types.coerce/2","doc":"Attempts to coerce a value to the specified type.\n\nCoercion is safe and predictable - it never raises exceptions and only\nperforms well-defined conversions.","ref":"Sinter.Types.html#coerce/2"},{"type":"function","title":"Parameters - Sinter.Types.coerce/2","doc":"* `type_spec` - The target type specification\n  * `value` - The value to coerce","ref":"Sinter.Types.html#coerce/2-parameters"},{"type":"function","title":"Returns - Sinter.Types.coerce/2","doc":"* `{:ok, coerced_value}` on successful coercion\n  * `{:error, [%Sinter.Error{}]}` on coercion failure","ref":"Sinter.Types.html#coerce/2-returns"},{"type":"function","title":"Examples - Sinter.Types.coerce/2","doc":"iex> Sinter.Types.coerce(:string, :hello)\n    {:ok, \"hello\"}\n\n    iex> Sinter.Types.coerce(:integer, \"42\")\n    {:ok, 42}\n\n    iex> {:error, [error]} = Sinter.Types.coerce(:integer, \"not a number\")\n    iex> error.code\n    :coercion","ref":"Sinter.Types.html#coerce/2-examples"},{"type":"type","title":"Sinter.Types.composite_type/0","doc":"","ref":"Sinter.Types.html#t:composite_type/0"},{"type":"type","title":"Sinter.Types.constraint/0","doc":"","ref":"Sinter.Types.html#t:constraint/0"},{"type":"type","title":"Sinter.Types.primitive_type/0","doc":"","ref":"Sinter.Types.html#t:primitive_type/0"},{"type":"function","title":"Sinter.Types.to_json_schema/1","doc":"Converts a type specification to JSON Schema format.\n\nThis function maps Sinter types to their JSON Schema equivalents,\nenabling JSON Schema generation for LLM providers.","ref":"Sinter.Types.html#to_json_schema/1"},{"type":"function","title":"Examples - Sinter.Types.to_json_schema/1","doc":"iex> Sinter.Types.to_json_schema(:string)\n    %{\"type\" => \"string\"}\n\n    iex> Sinter.Types.to_json_schema({:array, :integer})\n    %{\"type\" => \"array\", \"items\" => %{\"type\" => \"integer\"}}","ref":"Sinter.Types.html#to_json_schema/1-examples"},{"type":"type","title":"Sinter.Types.type_spec/0","doc":"","ref":"Sinter.Types.html#t:type_spec/0"},{"type":"function","title":"Sinter.Types.validate/3","doc":"Validates a value against a type specification.\n\nThis is the core validation function that all other validation ultimately uses.\nIt checks if a value matches the expected type and satisfies any constraints.","ref":"Sinter.Types.html#validate/3"},{"type":"function","title":"Parameters - Sinter.Types.validate/3","doc":"* `type_spec` - The type specification to validate against\n  * `value` - The value to validate\n  * `path` - The path for error reporting (default: [])","ref":"Sinter.Types.html#validate/3-parameters"},{"type":"function","title":"Returns - Sinter.Types.validate/3","doc":"* `{:ok, validated_value}` on success\n  * `{:error, [%Sinter.Error{}]}` on validation failure","ref":"Sinter.Types.html#validate/3-returns"},{"type":"function","title":"Examples - Sinter.Types.validate/3","doc":"iex> Sinter.Types.validate(:string, \"hello\", [])\n    {:ok, \"hello\"}\n\n    iex> Sinter.Types.validate(:integer, 42, [])\n    {:ok, 42}\n\n    iex> Sinter.Types.validate({:array, :string}, [\"a\", \"b\"], [])\n    {:ok, [\"a\", \"b\"]}\n\n    iex> {:error, [error]} = Sinter.Types.validate(:string, 123, [])\n    iex> error.code\n    :type","ref":"Sinter.Types.html#validate/3-examples"},{"type":"exception","title":"Sinter.ValidationError","doc":"Exception raised by `validate!` functions when validation fails.\n\nThis exception contains the validation errors that caused the failure,\nallowing the caller to access detailed error information programmatically.","ref":"Sinter.ValidationError.html"},{"type":"function","title":"Sinter.ValidationError.errors/1","doc":"Gets the validation errors from the exception.","ref":"Sinter.ValidationError.html#errors/1"},{"type":"function","title":"Sinter.ValidationError.exception/1","doc":"Creates a new ValidationError from a list of errors.","ref":"Sinter.ValidationError.html#exception/1"},{"type":"function","title":"Examples - Sinter.ValidationError.exception/1","doc":"iex> errors = [Sinter.Error.new([:name], :required, \"field is required\")]\n    iex> raise Sinter.ValidationError, errors: errors","ref":"Sinter.ValidationError.html#exception/1-examples"},{"type":"function","title":"Sinter.ValidationError.format/1","doc":"Formats the validation error for display.","ref":"Sinter.ValidationError.html#format/1"},{"type":"type","title":"Sinter.ValidationError.t/0","doc":"","ref":"Sinter.ValidationError.html#t:t/0"},{"type":"extras","title":"Sinter","doc":"# Sinter\n\n**Unified schema definition, validation, and JSON generation for Elixir**\n\n[![CI](https://github.com/nshkrdotcom/sinter/actions/workflows/ci.yml/badge.svg)](https://github.com/nshkrdotcom/sinter/actions/workflows/ci.yml)\n\nSinter is a focused, high-performance schema validation library for Elixir, designed specifically to power dynamic frameworks like DSPy. Born from the refactoring of Elixact, Sinter embodies the principle of **distillation** - transforming complex, overlapping APIs into a single, unified, and powerful validation engine.","ref":"readme.html"},{"type":"extras","title":"Why \"Sinter\"? - Sinter","doc":"**Sintering** is the process of fusing particles into a solid mass, mirroring how the library coalesces raw data into validated structures. It's a modern, technical term with an alchemical vibe that suggests strength and unification - ideal for a library that unifies compile-time and runtime schemas for dynamic frameworks.","ref":"readme.html#why-sinter"},{"type":"extras","title":"Philosophy - Sinter","doc":"Sinter follows the **\"One True Way\"** principle:\n- **One way** to define schemas (unified core engine)\n- **One way** to validate data (single validation pipeline)  \n- **One way** to generate JSON Schema (unified generator)\n- **Clear separation** of concerns (validation vs. transformation)\n\nThis architectural simplicity makes Sinter perfect for dynamic frameworks that need to create and modify schemas at runtime.","ref":"readme.html#philosophy"},{"type":"extras","title":"Features - Sinter","doc":"","ref":"readme.html#features"},{"type":"extras","title":"🎯 Unified Schema Definition - Sinter","doc":"```elixir\n# Runtime schema creation\nfields = [\n  {:name, :string, [required: true, min_length: 2]},\n  {:age, :integer, [optional: true, gt: 0]},\n  {:email, :string, [required: true, format: ~r/@/]}\n]\nschema = Sinter.Schema.define(fields, title: \"User Schema\")\n\n# Compile-time macro (uses same engine internally)\ndefmodule UserSchema do\n  import Sinter.Schema\n  \n  use_schema do\n    option :title, \"User Schema\"\n    option :strict, true\n    \n    field :name, :string, [required: true, min_length: 2]\n    field :age, :integer, [optional: true, gt: 0]  \n    field :email, :string, [required: true, format: ~r/@/]\n  end\nend\n```","ref":"readme.html#unified-schema-definition"},{"type":"extras","title":"⚡ Single Validation Pipeline - Sinter","doc":"```elixir\n# Works with any schema type\n{:ok, validated} = Sinter.Validator.validate(schema, data)\n\n# Optional post-validation hook for cross-field checks\nschema_with_hook = Sinter.Schema.define(fields, \n  post_validate: &check_business_rules/1\n)\n```","ref":"readme.html#single-validation-pipeline"},{"type":"extras","title":"🔄 Dynamic Schema Creation (Perfect for DSPy) - Sinter","doc":"```elixir\n# Create schemas on the fly - ideal for DSPy teleprompters\ndef create_program_signature(input_fields, output_fields) do\n  all_fields = input_fields ++ output_fields\n  Sinter.Schema.define(all_fields, title: \"DSPy Program Signature\")\nend\n\n# MIPRO-style dynamic optimization\noptimized_schema = create_program_signature.(\n  [{:question, :string, [required: true]}],\n  [{:answer, :string, [required: true]}, \n   {:confidence, :float, [required: true, gteq: 0.0, lteq: 1.0]}]\n)\n\n# Infer schemas from examples\nexamples = [\n  %{\"name\" => \"Alice\", \"age\" => 30},\n  %{\"name\" => \"Bob\", \"age\" => 25}\n]\ninferred_schema = Sinter.infer_schema(examples)\n\n# Merge schemas for composition\ninput_schema = Sinter.Schema.define([{:query, :string, [required: true]}])\noutput_schema = Sinter.Schema.define([{:answer, :string, [required: true]}])\nprogram_schema = Sinter.merge_schemas([input_schema, output_schema])\n```","ref":"readme.html#dynamic-schema-creation-perfect-for-dspy"},{"type":"extras","title":"📋 Unified JSON Schema Generation - Sinter","doc":"```elixir\n# Single function handles all schema types\njson_schema = Sinter.JsonSchema.generate(schema)\n\n# Provider-specific optimizations\nopenai_schema = Sinter.JsonSchema.for_provider(schema, :openai)\nanthropic_schema = Sinter.JsonSchema.for_provider(schema, :anthropic)\n```","ref":"readme.html#unified-json-schema-generation"},{"type":"extras","title":"🛠 Convenience Helpers - Sinter","doc":"```elixir\n# One-off type validation\n{:ok, 42} = Sinter.validate_type(:integer, \"42\", coerce: true)\n\n# Single field validation\n{:ok, \"john@example.com\"} = Sinter.validate_value(:email, :string, \n  \"john@example.com\", [format: ~r/@/])\n\n# Multiple value validation\n{:ok, results} = Sinter.validate_many([\n  {:string, \"hello\"},\n  {:integer, 42},\n  {:email, :string, \"test@example.com\", [format: ~r/@/]}\n])\n```","ref":"readme.html#convenience-helpers"},{"type":"extras","title":"Architecture - Sinter","doc":"Sinter's architecture follows the **distillation principle** - extracting the essential, powerful core from complex systems:\n\n```\n┌─────────────────────────────────────────────┐\n│              Sinter Core Engine             │\n├─────────────────────────────────────────────┤\n│  Sinter.Schema.define/2 (The Single Source) │\n├─────────────────────────────────────────────┤\n│         Sinter.Validator.validate/3         │\n├─────────────────────────────────────────────┤\n│         Sinter.JsonSchema.generate/2        │\n└─────────────────────────────────────────────┘\n           ▲                    ▲\n           │                    │\n    ┌──────────────┐    ┌──────────────┐\n    │  Built-in    │    │   Built-in   │\n    │ Compile-time │    │   Runtime    │\n    │  use_schema  │    │   Helpers    │\n    │   (macro)    │    │              │\n    └──────────────┘    └──────────────┘\n                               │\n                        ┌──────────────┐\n                        │validate_type │\n                        │validate_value│\n                        │validate_many │\n                        └──────────────┘\n```\n\n**All components shown are part of Sinter** - there are no external dependencies or systems required. The architecture shows:\n\n- **Core Engine**: The unified validation pipeline that all features use internally\n- **Compile-time macro**: `use_schema` - A convenience for defining schemas at compile time\n- **Runtime helpers**: `validate_type`, `validate_value`, `validate_many` - Convenience functions for quick validation tasks\n\nEverything flows through the same core engine, ensuring consistency and reliability across all usage patterns.","ref":"readme.html#architecture"},{"type":"extras","title":"Architectural Benefits - Sinter","doc":"","ref":"readme.html#architectural-benefits"},{"type":"extras","title":"✅ From Elixact's Complexity - Sinter","doc":"- 5 different ways to define schemas → **1 unified way**\n- 4 validation modules → **1 validation engine**  \n- 3 JSON Schema modules → **1 generator**\n- Complex transformation pipeline → **Simple validation + explicit app logic**","ref":"readme.html#from-elixact-s-complexity"},{"type":"extras","title":"✅ Perfect for DSPy/Dynamic Frameworks - Sinter","doc":"- Runtime schema creation without architectural friction\n- Clean separation: validation (Sinter) vs. transformation (your app)\n- No \"magic\" - every step in your program is explicit and optimizable\n- Unified API means less cognitive overhead","ref":"readme.html#perfect-for-dspy-dynamic-frameworks"},{"type":"extras","title":"✅ Performance Benefits - Sinter","doc":"- Fewer abstraction layers = less overhead\n- Single validation path = easier optimization\n- No complex transformation pipeline = cleaner performance profile","ref":"readme.html#performance-benefits"},{"type":"extras","title":"Installation - Sinter","doc":"Add `sinter` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:sinter, \"~> 0.0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - Sinter","doc":"```elixir\n# 1. Define a schema\nfields = [\n  {:name, :string, [required: true]},\n  {:age, :integer, [optional: true, gt: 0]}\n]\nschema = Sinter.Schema.define(fields)\n\n# 2. Validate data  \n{:ok, validated} = Sinter.Validator.validate(schema, %{\n  name: \"Alice\",\n  age: 30\n})\n\n# 3. Generate JSON Schema\njson_schema = Sinter.JsonSchema.generate(schema)\n```","ref":"readme.html#quick-start"},{"type":"extras","title":"🚀 Try the Examples - Sinter","doc":"Explore comprehensive examples covering all functionality:\n\n```bash\n# Run all examples with detailed reporting\ncd examples && elixir run_all.exs\n\n# Or run individual examples\ncd examples\nelixir basic_usage.exs           # Core functionality\nelixir readme_comprehensive.exs  # All README examples  \nelixir dspy_integration.exs      # DSPy framework patterns\n```\n\nSee [`examples/README.md`](examples/README.md) for detailed documentation.","ref":"readme.html#try-the-examples"},{"type":"extras","title":"Migration from Elixact - Sinter","doc":"Sinter provides a clean migration path from Elixact's complex APIs:\n\n| Elixact (Before) | Sinter (After) |\n|------------------|----------------|\n| `Elixact.Runtime.create_schema` | `Sinter.Schema.define` |\n| `EnhancedValidator.validate` | `Sinter.Validator.validate` |  \n| `TypeAdapter.validate` | `Sinter.validate_type` |\n| `Wrapper.wrap_and_validate` | `Sinter.validate_value` |\n| Multiple JSON Schema modules | `Sinter.JsonSchema.generate` |","ref":"readme.html#migration-from-elixact"},{"type":"extras","title":"Key Design Decisions - Sinter","doc":"### **Validation ≠ Transformation**\nSinter validates data structure and constraints. Data transformation is your application's responsibility, keeping your program logic explicit and optimizable.\n\n### **Runtime-First Design**  \nWhile compile-time macros are supported, the core engine is built for runtime schema creation - perfect for dynamic frameworks.\n\n### **Proven Foundation**\nSinter leverages battle-tested Elixir libraries:\n- **Jason** - Fast, reliable JSON parsing\n- **ExUnit** - Comprehensive testing framework\n- **Dialyzer** - Static type analysis","ref":"readme.html#key-design-decisions"},{"type":"extras","title":"Documentation - Sinter","doc":"- [API Reference](https://hexdocs.pm/sinter)\n- [Examples Directory](examples/README.md) - Comprehensive working examples","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - Sinter","doc":"We welcome contributions! Please:\n\n1. Fork the repository\n2. Create a feature branch (`git checkout -b feature/amazing-feature`)\n3. Make your changes with tests\n4. Run the full test suite (`mix test`)\n5. Submit a pull request","ref":"readme.html#contributing"},{"type":"extras","title":"License - Sinter","doc":"MIT","ref":"readme.html#license"},{"type":"extras","title":"Acknowledgments - Sinter","doc":"Sinter is the distilled essence of [Elixact](https://github.com/your-org/elixact), refactored specifically for dynamic framework needs. Special thanks to the Elixact contributors and the broader Elixir validation ecosystem.\n\n---\n\n*\"In the furnace of simplicity, complexity becomes strength.\"*","ref":"readme.html#acknowledgments"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.0.1] - 2025-07-05","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release of Sinter\n- Unified schema definition, validation, and JSON generation for Elixir\n- Core features:\n  - Unified schema definition with `Sinter.Schema.define/2`\n  - Single validation pipeline with `Sinter.Validator.validate/3`\n  - JSON Schema generation with `Sinter.JsonSchema.generate/2`\n  - Dynamic schema creation for frameworks like DSPy\n  - Schema inference from examples\n  - Schema merging for composition\n  - Provider-specific JSON Schema optimizations\n- Convenience helpers:\n  - `validate_type/3` for one-off type validation\n  - `validate_value/4` for single field validation\n  - `validate_many/1` for multiple value validation\n- Compile-time macro support with `use_schema`\n- Comprehensive example suite\n- Full test coverage\n- Documentation and API reference\n\n[0.0.1]: https://github.com/nshkrdotcom/sinter/releases/tag/v0.0.1","ref":"changelog.html#added"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 NSHkr\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}